@@ -1,7 +1,7 @@
-#' Main function to carry out immunaut analysis
+#' Main function to carry out Immunaut Analysis
 #'
 #' This function performs clustering and dimensionality reduction analysis on a dataset using user-defined settings.
-#' It handles various pre-processing steps, dimensionality reduction via t-SNE, multiple clustering methods, and
+#' It handles various preprocessing steps, dimensionality reduction via t-SNE, multiple clustering methods, and
 #' generates associated plots based on user-defined or default settings.
 #'
 #' @param dataset A data frame representing the dataset on which the analysis will be performed. The dataset must 
@@ -11,50 +11,78 @@
 #' \describe{
 #'   \item{fileHeader}{A data frame mapping the original column names to remapped column names. Used for t-SNE input preparation.}
 #'   \item{selectedColumns}{Character vector of columns to be used for the analysis. Defaults to NULL.}
-#'   \item{cutOffColumnSize}{Numeric, the maximum size of the dataset in terms of columns. Defaults to 50,000.}
+#'   \item{cutOffColumnSize}{Numeric. The maximum size of the dataset in terms of columns. Defaults to 50,000.}
 #'   \item{excludedColumns}{Character vector of columns to exclude from the analysis. Defaults to NULL.}
 #'   \item{groupingVariables}{Character vector of columns to use for grouping the data during analysis. Defaults to NULL.}
 #'   \item{colorVariables}{Character vector of columns to use for coloring in the plots. Defaults to NULL.}
-#'   \item{preProcessDataset}{Character vector of pre-processing methods to apply (e.g., scaling, normalization). Defaults to NULL.}
-#'   \item{fontSize}{Numeric, font size for plots. Defaults to 12.}
-#'   \item{pointSize}{Numeric, size of points in plots. Defaults to 1.5.}
-#'   \item{theme}{Character, the ggplot2 theme to use (e.g., "theme_gray"). Defaults to "theme_gray".}
-#'   \item{colorPalette}{Character, color palette for plots (e.g., "RdPu"). Defaults to "RdPu".}
-#'   \item{aspect_ratio}{Numeric, the aspect ratio of plots. Defaults to 1.}
-#'   \item{clusterType}{Character, the clustering method to use. Options are "Louvain", "Hierarchical", "Mclust", "Density". Defaults to "Louvain".}
-#'   \item{removeNA}{Logical, whether to remove rows with NA values. Defaults to FALSE.}
-#'   \item{datasetAnalysisGrouped}{Logical, whether to perform grouped dataset analysis. Defaults to FALSE.}
-#'   \item{plot_size}{Numeric, the size of the plot. Defaults to 12.}
-#'   \item{knn_clusters}{Numeric, the number of clusters for KNN-based clustering. Defaults to 250.}
-#'   \item{perplexity}{Numeric, the perplexity parameter for t-SNE. Defaults to NULL (automatically determined).}
-#'   \item{exaggeration_factor}{Numeric, the exaggeration factor for t-SNE. Defaults to NULL.}
-#'   \item{max_iter}{Numeric, the maximum number of iterations for t-SNE. Defaults to NULL.}
-#'   \item{theta}{Numeric, the Barnes-Hut approximation parameter for t-SNE. Defaults to NULL.}
-#'   \item{eta}{Numeric, the learning rate for t-SNE. Defaults to NULL.}
-#'   \item{clustLinkage}{Character, linkage method for hierarchical clustering. Defaults to "ward.D2".}
-#'   \item{clustGroups}{Numeric, the number of groups for hierarchical clustering. Defaults to 9.}
-#'   \item{distMethod}{Character, distance metric for clustering. Defaults to "euclidean".}
-#'   \item{minPtsAdjustmentFactor}{Numeric, adjustment factor for the minimum points in DBSCAN clustering. Defaults to 1.}
-#'   \item{epsQuantile}{Numeric, quantile to compute the epsilon parameter for DBSCAN clustering. Defaults to 0.9.}
-#'   \item{assignOutliers}{Logical, whether to assign outliers in the clustering step. Defaults to TRUE.}
-#'   \item{excludeOutliers}{Logical, whether to exclude outliers from clustering. Defaults to TRUE.}
-#'   \item{legendPosition}{Character, position of the legend in plots (e.g., "right", "bottom"). Defaults to "right".}
-#'   \item{datasetAnalysisClustLinkage}{Character, linkage method for dataset-level analysis. Defaults to "ward.D2".}
-#'   \item{datasetAnalysisType}{Character, type of dataset analysis (e.g., "heatmap"). Defaults to "heatmap".}
-#'   \item{datasetAnalysisRemoveOutliersDownstream}{Logical, whether to remove outliers during downstream dataset analysis (ML). Defaults to FALSE.}
-#'   \item{datasetAnalysisSortColumn}{Character, the column used to sort dataset analysis results. Defaults to "cluster".}
-#'   \item{datasetAnalysisClustOrdering}{Numeric, the order of clusters for analysis. Defaults to 1.}
-#'   \item{anyNAValues}{Logical, whether the dataset contains NA values. Defaults to FALSE.}
-#'   \item{categoricalVariables}{Logical, whether the dataset contains categorical variables. Defaults to FALSE.}
+#'   \item{preProcessDataset}{Character vector of preprocessing methods to apply (e.g., scaling, normalization). Defaults to NULL.}
+#'   \item{fontSize}{Numeric. Font size for plots. Defaults to 12.}
+#'   \item{pointSize}{Numeric. Size of points in plots. Defaults to 1.5.}
+#'   \item{theme}{Character. The ggplot2 theme to use (e.g., "theme_gray"). Defaults to "theme_gray".}
+#'   \item{colorPalette}{Character. Color palette for plots (e.g., "RdPu"). Defaults to "RdPu".}
+#'   \item{aspect_ratio}{Numeric. The aspect ratio of plots. Defaults to 1.}
+#'   \item{clusterType}{Character. The clustering method to use. Options are "Louvain", "Hierarchical", "Mclust", "Density". Defaults to "Louvain".}
+#'   \item{removeNA}{Logical. Whether to remove rows with NA values. Defaults to FALSE.}
+#'   \item{datasetAnalysisGrouped}{Logical. Whether to perform grouped dataset analysis. Defaults to FALSE.}
+#'   \item{plot_size}{Numeric. The size of the plot. Defaults to 12.}
+#'   \item{knn_clusters}{Numeric. The number of clusters for KNN-based clustering. Defaults to 250.}
+#'   \item{perplexity}{Numeric. The perplexity parameter for t-SNE. Defaults to NULL (automatically determined).}
+#'   \item{exaggeration_factor}{Numeric. The exaggeration factor for t-SNE. Defaults to NULL.}
+#'   \item{max_iter}{Numeric. The maximum number of iterations for t-SNE. Defaults to NULL.}
+#'   \item{theta}{Numeric. The Barnes-Hut approximation parameter for t-SNE. Defaults to NULL.}
+#'   \item{eta}{Numeric. The learning rate for t-SNE. Defaults to NULL.}
+#'   \item{clustLinkage}{Character. Linkage method for hierarchical clustering. Defaults to "ward.D2".}
+#'   \item{clustGroups}{Numeric. The number of groups for hierarchical clustering. Defaults to 9.}
+#'   \item{distMethod}{Character. Distance metric for clustering. Defaults to "euclidean".}
+#'   \item{minPtsAdjustmentFactor}{Numeric. Adjustment factor for the minimum points in DBSCAN clustering. Defaults to 1.}
+#'   \item{epsQuantile}{Numeric. Quantile to compute the epsilon parameter for DBSCAN clustering. Defaults to 0.9.}
+#'   \item{assignOutliers}{Logical. Whether to assign outliers in the clustering step. Defaults to TRUE.}
+#'   \item{excludeOutliers}{Logical. Whether to exclude outliers from clustering. Defaults to TRUE.}
+#'   \item{legendPosition}{Character. Position of the legend in plots (e.g., "right", "bottom"). Defaults to "right".}
+#'   \item{datasetAnalysisClustLinkage}{Character. Linkage method for dataset-level analysis. Defaults to "ward.D2".}
+#'   \item{datasetAnalysisType}{Character. Type of dataset analysis (e.g., "heatmap"). Defaults to "heatmap".}
+#'   \item{datasetAnalysisRemoveOutliersDownstream}{Logical. Whether to remove outliers during downstream dataset analysis (e.g., machine learning). Defaults to FALSE.}
+#'   \item{datasetAnalysisSortColumn}{Character. The column used to sort dataset analysis results. Defaults to "cluster".}
+#'   \item{datasetAnalysisClustOrdering}{Numeric. The order of clusters for analysis. Defaults to 1.}
+#'   \item{anyNAValues}{Logical. Whether the dataset contains NA values. Defaults to FALSE.}
+#'   \item{categoricalVariables}{Logical. Whether the dataset contains categorical variables. Defaults to FALSE.}
+#'   \item{resolution_increments}{Numeric vector. The resolution increments to be used for Louvain clustering. Defaults to \code{c(0.01, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5)}.}
+#'   \item{min_modularities}{Numeric vector. The minimum modularities to test for clustering. Defaults to \code{c(0.4, 0.5, 0.6, 0.7, 0.8, 0.85, 0.9)}.}
+#'   \item{target_clusters_range}{Numeric vector. The range of acceptable clusters to identify. Defaults to \code{c(3, 6)}.}
+#'   \item{pickBestClusterMethod}{Character. The method to use for picking the best clustering result ("Modularity", "Silhouette", or "SIMON"). Defaults to "Modularity".}
+#'   \item{weights}{List. Weights for evaluating clusters based on \code{AUROC}, \code{modularity}, and \code{silhouette}. Defaults to \code{list(AUROC = 0.5, modularity = 0.3, silhouette = 0.2)}. These weights are applied to help choose the most relevant clusters based on user goals:
+#'   \describe{
+#'     \item{\code{AUROC}}{Weight for predictive performance (area under the receiver operating characteristic curve). Prioritize this when predictive accuracy is the main goal. For predictive analysis, a recommended configuration could be \code{list(AUROC = 0.8, modularity = 0.1, silhouette = 0.1)}.}
+#'     \item{\code{modularity}}{Weight for modularity score, which indicates the strength of clustering. Higher modularity suggests that clusters are well-separated. To prioritize well-separated clusters, use a configuration like \code{list(AUROC = 0.4, modularity = 0.4, silhouette = 0.2)}.}
+#'     \item{\code{silhouette}}{Weight for silhouette score, a measure of cohesion within clusters. Useful when cluster cohesion and interpretability are desired. For balanced clusters, a suggested configuration is \code{list(AUROC = 0.4, modularity = 0.3, silhouette = 0.3)}.}
+#'   }}
 #' }
 #'
-#' @importFrom dplyr select filter group_by summarize_all rename
+#' @importFrom dplyr select filter group_by summarise_all rename
 #' @importFrom rlang is_null
 #' @importFrom stats hclust dist na.omit
 #' @importFrom mclust Mclust
 #' @importFrom fpc dbscan
 #'
-#' @return A data frame with cluster assignments added as a new column. If NULL, warning messages will be printed for any erroneous input parameters.
+#' @return A list containing the following:
+#' \itemize{
+#'   \item \code{tsne_calc}: The t-SNE results object.
+#'   \item \code{tsne_clust}: The clustering results.
+#'   \item \code{dataset}: A list containing the original dataset, the preprocessed dataset, and a dataset with machine learning-ready data.
+#'   \item \code{clusters}: The final cluster assignments.
+#'   \item \code{settings}: The list of settings used for the analysis.
+#' }
+#'
+#' @examples
+#' \donttest{
+#'   data <- matrix(runif(2000), ncol=20)
+#'   settings <- list(clusterType = "Louvain", 
+#'   resolution_increments = c(0.05, 0.1), 
+#'   min_modularities = c(0.3, 0.5))
+#'   result <- immunaut(data.frame(data), settings)
+#'   print(result$clusters)
+#' }
+#'
 #' @export
 immunaut <- function(dataset, settings = list()){
 
@@ -136,10 +164,51 @@ immunaut <- function(dataset, settings = list()){
         settings$clusterType <- "Louvain"
     }
 
+    ## Louvain Specific START
+    if(is_var_empty(settings$resolution_increments) == TRUE){
+        settings$resolution_increments <-  c(0.01, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5)
+        #settings$resolution_increments <- c(0.01, 0.1, 0.2, 0.3, 0.4)
+    }
+
+    if(is_var_empty(settings$min_modularities) == TRUE){
+        settings$min_modularities <- c(0.4, 0.5, 0.6, 0.7, 0.8, 0.85, 0.9)
+        #settings$min_modularities <- c(0.5, 0.6, 0.7, 0.8) 
+    }
+
+    if(is_var_empty(settings$target_clusters_range) == TRUE){
+        settings$target_clusters_range <- c(3, 6)
+    }
+
+    if(is_var_empty(settings$pickBestClusterMethod) == TRUE){
+        settings$pickBestClusterMethod <- "Modularity" ## Modularity, Silhouette, Overall, SIMON
+    }
+
+    if(is_var_empty(settings$selectedPartitionSplit) == TRUE){
+        settings$selectedPartitionSplit <- 0.7
+    }
+
+    if(is_var_empty(settings$selectedPackages) == TRUE){
+        settings$selectedPackages <- c("rf", "RRF", "RRFglobal", "gcvEarth", "cforest", "nb")
+    }
+
+    if(is_var_empty(settings$trainingTimeout) == TRUE){
+        settings$trainingTimeout <- 360
+    }
+
+    if(is_var_empty(settings$weights) == TRUE){
+        settings$weights <- list(AUROC = 0.5, modularity = 0.3, silhouette = 0.2)
+    }
+
+    ## Louvain Specific END
+
     if(is_var_empty(settings$removeNA) == TRUE){
         settings$removeNA = FALSE
     }
 
+    if(is_var_empty(settings$seed) == TRUE){
+        settings$seed = 1337
+    }
+
     if(is_var_empty(settings$datasetAnalysisGrouped) == TRUE){
         settings$datasetAnalysisGrouped = FALSE
     }
@@ -278,7 +347,7 @@ immunaut <- function(dataset, settings = list()){
     if (is.null(vars_to_cast)){
         vars_to_cast <- character(0)
     }else{
-        print(paste("==> Casting to NA: ", vars_to_cast))
+        message(paste("==> Casting to NA: ", vars_to_cast))
     }
 
     num_test <- dataset_filtered %>% select(where(is.numeric))
@@ -296,27 +365,57 @@ immunaut <- function(dataset, settings = list()){
     	preProcessMapping <- preProcessResample(dataset_filtered, 
     	    settings$preProcessDataset, 
     	    settings$groupingVariables, 
-    	    settings$groupingVariables)
+    	    settings$groupingVariables,
+            settings)
 
        dataset_filtered <- preProcessMapping$datasetData
    }
 
 	# 3. remove NA is any left
     if(settings$removeNA == TRUE){
-        print(paste0("=====> Removing NA Values"))
+        message("===> INFO: Removing NA Values")
         dataset_filtered <- na.omit(dataset_filtered)
     }
 
 	# 4. calculate_tsne
-    set.seed(1337)
+    set.seed(settings$seed)
     tsne_calc <- calculate_tsne(dataset_filtered, settings)
     
 
 	# 5. cluster tsne
-    print(paste0("===> Clustering using", settings$clusterType))
-    set.seed(1337)
+    message("===> INFO: Clustering using", settings$clusterType)
+    set.seed(settings$seed)
     if(settings$clusterType == "Louvain"){
-       tsne_clust <- cluster_tsne_knn_louvain(tsne_calc$info.norm, tsne_calc$tsne.norm, settings)
+
+        tsne_clust <- list()
+        iteration <- 1
+        for (res_increment in settings$resolution_increments) {
+            for (min_modularity in settings$min_modularities) {
+                tmp <- cluster_tsne_knn_louvain(tsne_calc$info.norm, tsne_calc$tsne.norm, settings, res_increment, min_modularity)
+
+                if(tmp$num_clusters < min(settings$target_clusters_range) && tmp$num_clusters > max(settings$target_clusters_range)){
+                    message("===> INFO: Skipping cluster with ", tmp$num_clusters, " clusters")
+                    next
+                }
+                
+                tsne_clust[[iteration]] <- tmp
+                iteration <- iteration + 1
+            }
+        }
+
+        if(settings$pickBestClusterMethod == "Modularity"){
+            tsne_clust <- pick_best_cluster_modularity(tsne_clust)
+        }else if(settings$pickBestClusterMethod == "Silhouette"){
+            tsne_clust <- pick_best_cluster_silhouette(tsne_clust)
+        }else if(settings$pickBestClusterMethod == "Overall"){
+            tsne_clust <- pick_best_cluster_overall(tsne_clust, tsne_calc)
+        }else if(settings$pickBestClusterMethod == "SIMON"){
+            best_cluster <- pick_best_cluster_simon(dataset, tsne_clust, tsne_calc, settings)
+            tsne_clust <- best_cluster$tsne_clust
+        }else{
+            tsne_clust <- pick_best_cluster_modularity(tsne_clust)
+        }
+
     }else if(settings$clusterType == "Hierarchical"){
        tsne_clust <- cluster_tsne_hierarchical(tsne_calc$info.norm, tsne_calc$tsne.norm, settings)
     }else if(settings$clusterType == "Mclust"){
@@ -324,7 +423,34 @@ immunaut <- function(dataset, settings = list()){
     }else if(settings$clusterType == "Density"){
        tsne_clust <- cluster_tsne_density(tsne_calc$info.norm, tsne_calc$tsne.norm, settings)
     }else{
-       tsne_clust <- cluster_tsne_knn_louvain(tsne_calc$info.norm, tsne_calc$tsne.norm, settings)
+        tsne_clust <- list()
+        iteration <- 1
+        for (res_increment in settings$resolution_increments) {
+            for (min_modularity in settings$min_modularities) {
+                tmp <- cluster_tsne_knn_louvain(tsne_calc$info.norm, tsne_calc$tsne.norm, settings, res_increment, min_modularity)
+
+                if(tmp$num_clusters < min(settings$target_clusters_range) && tmp$num_clusters > max(settings$target_clusters_range)){
+                    message("===> INFO: Skipping cluster with ", tmp$num_clusters, " clusters")
+                    next
+                }
+                
+                tsne_clust[[iteration]] <- tmp
+                iteration <- iteration + 1
+            }
+        }
+
+        if(settings$pickBestClusterMethod == "Modularity"){
+            tsne_clust <- pick_best_cluster_modularity(tsne_clust)
+        }else if(settings$pickBestClusterMethod == "Silhouette"){
+            tsne_clust <- pick_best_cluster_silhouette(tsne_clust)
+        }else if(settings$pickBestClusterMethod == "Overall"){
+            tsne_clust <- pick_best_cluster_overall(tsne_clust, tsne_calc)
+        }else if(settings$pickBestClusterMethod == "SIMON"){
+            best_cluster <- pick_best_cluster_simon(dataset, tsne_clust, tsne_calc, settings)
+            tsne_clust <- best_cluster$tsne_clust
+        }else{
+            tsne_clust <- pick_best_cluster_modularity(tsne_clust)
+        }
     }
 
     ## Get clusters from tsne_clust$info.norm[["pandora_cluster"]] and add it to original dataset in "dataset" variable
